# In order so one can just call from ensenso_nxlib import NxLibException (for example)
# flake8: noqa
from .exception import NxLibException, NxLibError
from .command import NxLibCommand
from .item import NxLibItem



# -*- coding: utf-8 -*-
from .exception import NxLibException, NxLibError

from ctypes import *

import ensenso_nxlib.helper as helper

__all__ = [
    "close_tcp_port",
    "connect",
    "disconnect",
    "erase",
    "finalize",
    "get_binary",
    "get_binary_info",
    "get_bool",
    "get_count",
    "get_debug_buffer",
    "get_debug_messages",
    "get_double",
    "get_int",
    "get_json",
    "get_json_meta",
    "get_name",
    "get_string",
    "get_type",
    "initialize",
    "is_current_lib_remote",
    "load_lib",
    "load_remote_lib",
    "make_unique_item",
    "open_tcp_port",
    "set_binary",
    "set_binary_formatted",
    "set_bool",
    "set_double",
    "set_int",
    "set_json",
    "set_null",
    "set_string",
    "translate_error_code",
    "wait_for_bool_value",
    "wait_for_change",
    "wait_for_double_value",
    "wait_for_int_value",
    "wait_for_string_value",
    "wait_for_type"
]

NXLIB_INTERNAL_ERROR = 7
NXLIB_OPERATION_SUCCEEDED = 0

NX_ERRORS = ["NxLibOperationSucceeded",
             "NxLibCannotCreateItem",
             "NxLibCouldNotInterpretJsonText",
             "NxLibItemInexistent",
             "",
             "NxLibNoDebugData",
             "NxLibCouldNotOpenPort",
             "NxLibInternalError",
             "NxLibTimeout",
             "NxLibNotConnected",
             "NxLibMethodInvalid",
             "NxLibBadRequest",
             "",
             "NxLibItemTypeNotCompatible",
             "NxLibInvalidBufferSize",
             "NxLibBufferTooSmall",
             "NxLibBufferNotDivisibleByElementSize",
             "NxLibExecutionFailed",
             "NxLibDebugMessageOverflow",
             "",
             "",
             "",
             "NxLibConnectionNotCompatible",
             "NxLibInitializationNotAllowed",
             "NxLibNestingLimitReached",
             "NxLibNoOpenProfileBlock"]


class _Nxlib():
    def __init__(self):
        self.is_remote = False
        self.lib_object = None

    def __del__(self):
        if self.is_remote:
            disconnect()
        else:
            finalize()
        if self.lib_object:
            del self.lib_object


# testing flag
__nx_testing__ = False

# global placeholder for the nxlib_instance
_nxlib = _Nxlib()


def _get_lib():
    # will default load the normal nxlib, if there has been no load_remote_lib beforehand!
    global _nxlib
    if _nxlib.lib_object is None:
        if _nxlib.is_remote:
            load_remote_lib()
        else:
            load_lib()
    return _nxlib.lib_object


def is_current_lib_remote():
    # For Testing and debug purpose
    global _nxlib
    return _nxlib.is_remote


def _lib_is_remote():
    global _nxlib
    return _nxlib.is_remote


def load_lib(path=None):
    global _nxlib
    if path is None:
        _nxlib.lib_object = CDLL(helper.get_lib_name())
    else:
        _nxlib.lib_object = CDLL(path)
    _nxlib.is_remote = False
    if _nxlib.lib_object is None:
        raise NxLibError("Could not load shared library")


def load_remote_lib(path=None):
    global _nxlib
    if path is None:
        _nxlib.lib_object = CDLL(helper.get_lib_name(True))
    else:
        _nxlib.lib_object = CDLL(path)
    _nxlib.is_remote = True
    if _nxlib.lib_object is None:
        raise NxLibError("Could not load shared remote library")


def check_return_code(error_code):
    if error_code != NXLIB_OPERATION_SUCCEEDED:
        raise NxLibException("", error_code)


def _set(f, path, value):
    path = helper.fix_string_encoding(path)
    error_code = c_int32(0)
    f(byref(error_code), path, value)
    return error_code.value


def set_null(path):
    path = helper.fix_string_encoding(path)
    f = _get_lib().nxLibSetNull
    f.argtypes = [POINTER(c_int32), c_char_p]
    error_code = c_int32(0)
    f(byref(error_code), path)
    return error_code.value


def set_int(path, value):
    f = _get_lib().nxLibSetInt
    f.argtypes = [POINTER(c_int32), c_char_p, c_int32]
    return _set(f, path, value)


def set_double(path, value):
    f = _get_lib().nxLibSetDouble
    f.argtypes = [POINTER(c_int32), c_char_p, c_double]
    return _set(f, path, value)


def set_bool(path, value):
    f = _get_lib().nxLibSetBool
    f.argtypes = [POINTER(c_int32), c_char_p, c_int32]
    return _set(f, path, value)


def set_string(path, value):
    value = helper.fix_string_encoding(value)
    f = _get_lib().nxLibSetString
    f.argtypes = [POINTER(c_int32), c_char_p, c_char_p]
    return _set(f, path, value)


def set_json(path, value, only_writeable_nodes=False):
    path = helper.fix_string_encoding(path)
    value = helper.fix_string_encoding(value)
    f = _get_lib().nxLibSetJson
    f.argtypes = [POINTER(c_int32), c_char_p, c_char_p, c_int32]
    error_code = c_int32(0)
    f(byref(error_code), path, value, only_writeable_nodes)
    return error_code.value


def set_binary(path, buffer, buffer_size):
    path = helper.fix_string_encoding(path)
    f = _get_lib().nxLibSetBinary
    f.argtypes = [POINTER(c_int32), c_char_p, POINTER(c_void_p), c_int32]
    buffer = cast(buffer, POINTER(c_void_p))
    error_code = c_int32(0)
    f(byref(error_code), path, buffer, buffer_size)
    return error_code.value


def set_binary_formatted(path, buffer, width, height, channel_count, bytes_per_element, is_float):
    path = helper.fix_string_encoding(path)
    f = _get_lib().nxLibSetBinaryFormatted
    f.argtypes = [POINTER(c_int32), c_char_p, POINTER(
        c_void_p), c_int32, c_int32, c_int32, c_int32, c_int32]
    buffer = cast(buffer, POINTER(c_void_p))
    error_code = c_int32(0)
    f(byref(error_code), path, buffer, width, height,
      channel_count, bytes_per_element, is_float)
    return error_code.value


def get_binary(path, destination_buffer, buffer_size):
    path = helper.fix_string_encoding(path)
    f = _get_lib().nxLibGetBinary
    f.argtypes = [POINTER(c_int32), c_char_p, POINTER(
        c_void_p), c_int32, POINTER(c_int32), POINTER(c_double)]
    error_code = c_int32(0)
    destination_buffer = cast(destination_buffer, POINTER(c_void_p))
    # destination_buffer = c_void_p(buffer_size) #(c_ubyte * buffer_size)()
    bytes_copied = c_int32(0)
    timestamp = c_double(0)

    f(byref(error_code), path, destination_buffer,
      buffer_size, byref(bytes_copied), byref(timestamp))

    return bytes_copied.value, timestamp.value, error_code.value


def get_binary_info(path):
    path = helper.fix_string_encoding(path)
    f = _get_lib().nxLibGetBinaryInfo
    f.argtypes = [POINTER(c_int32), c_char_p, POINTER(c_int32), POINTER(c_int32), POINTER(c_int32),
                  POINTER(c_int32), POINTER(c_int32), POINTER(c_double)]

    error_code = c_int32(0)
    width = c_int32(0)
    height = c_int32(0)
    channel_count = c_int32(0)
    bytes_per_element = c_int32(0)
    is_float = c_int32(0)
    timestamp = c_double(0)

    f(byref(error_code), path, byref(width), byref(height), byref(channel_count), byref(bytes_per_element),
      byref(is_float), byref(timestamp))

    return (width.value, height.value, channel_count.value, bytes_per_element.value,
            is_float.value == 1, timestamp.value, error_code.value)


def _get(f, path):
    path = helper.fix_string_encoding(path)
    error_code = c_int32(0)
    result = f(byref(error_code), path)
    return result, error_code.value


def get_type(path):
    f = _get_lib().nxLibGetType
    f.restype = c_int32
    f.argtypes = [POINTER(c_int32), c_char_p]
    return _get(f, path)


def get_int(path):
    f = _get_lib().nxLibGetInt
    f.restype = c_int32
    f.argtypes = [POINTER(c_int32), c_char_p]
    return _get(f, path)


def get_bool(path):
    f = _get_lib().nxLibGetBool
    f.restype = c_int32
    f.argtypes = [POINTER(c_int32), c_char_p]
    b, error_code = _get(f, path)
    return b == 1, error_code


def get_count(path):
    f = _get_lib().nxLibGetCount
    f.restype = c_int32
    f.argtypes = [POINTER(c_int32), c_char_p]
    return _get(f, path)


def get_double(path):
    f = _get_lib().nxLibGetDouble
    f.restype = c_double
    f.argtypes = [POINTER(c_int32), c_char_p]
    return _get(f, path)


def get_string(path):
    f = _get_lib().nxLibGetString
    f.restype = c_char_p
    f.argtypes = [POINTER(c_int32), c_char_p]
    s, error_code = _get(f, path)
    if s is not None:
        s = s.decode()
    return s, error_code


def get_name(path):
    f = _get_lib().nxLibGetName
    f.restype = c_char_p
    f.argtypes = [POINTER(c_int32), c_char_p]
    s, error_code = _get(f, path)
    if s is not None:
        s = s.decode()
    return s, error_code


def get_json(path, pretty_print, number_precision, scientific_number_format):
    path = helper.fix_string_encoding(path)
    f = _get_lib().nxLibGetJson
    f.restype = c_char_p
    f.argtypes = [POINTER(c_int32), c_char_p, c_int32, c_int32, c_int32]
    error_code = c_int32(0)
    result = f(byref(error_code), path, pretty_print,
               number_precision, scientific_number_format)
    if result is not None:
        result = result.decode()
    return result, error_code.value


def get_json_meta(path, num_levels, pretty_print, number_precision, scientific_number_format):
    path = helper.fix_string_encoding(path)

    f = _get_lib().nxLibGetJsonMeta
    f.restype = c_char_p
    f.argtypes = [POINTER(c_int32), c_char_p, c_int32,
                  c_int32, c_int32, c_int32]
    error_code = c_int32(0)
    result = f(byref(error_code), path, num_levels, pretty_print,
               number_precision, scientific_number_format)
    if result is not None:
        result = result.decode()
    return result, error_code.value


def erase(path):
    path = helper.fix_string_encoding(path)
    f = _get_lib().nxLibErase
    f.argtypes = [POINTER(c_int32), c_char_p]
    error_code = c_int32(0)
    f(byref(error_code), path)
    return error_code.value


def wait_for_change(path):
    path = helper.fix_string_encoding(path)
    f = _get_lib().nxLibWaitForChange
    f.argtypes = [POINTER(c_int32), c_char_p]
    error_code = c_int32(0)
    f(byref(error_code), path)
    return error_code.value


def wait_for_type(path, awaited_type, wait_for_equal):
    path = helper.fix_string_encoding(path)
    f = _get_lib().nxLibWaitForType
    f.argtypes = [POINTER(c_int32), c_char_p, c_int32, c_int32]
    error_code = c_int32(0)
    f(byref(error_code), path, awaited_type, wait_for_equal)
    return error_code.value


def wait_for_int_value(path, value, wait_for_equal):
    path = helper.fix_string_encoding(path)
    f = _get_lib().nxLibWaitForIntValue
    f.argtypes = [POINTER(c_int32), c_char_p, c_int32, c_int32]
    error_code = c_int32(0)
    f(byref(error_code), path, value, wait_for_equal)
    return error_code.value


def wait_for_string_value(path, value, wait_for_equal):
    path = helper.fix_string_encoding(path)
    value = helper.fix_string_encoding(value)
    f = _get_lib().nxLibWaitForStringValue
    f.argtypes = [POINTER(c_int32), c_char_p, c_char_p, c_int32]
    error_code = c_int32(0)
    f(byref(error_code), path, value, wait_for_equal)
    return error_code.value


def wait_for_double_value(path, value, wait_for_equal):
    path = helper.fix_string_encoding(path)
    f = _get_lib().nxLibWaitForDoubleValue
    f.argtypes = [POINTER(c_int32), c_char_p, c_double, c_int32]
    error_code = c_int32(0)
    f(byref(error_code), path, value, wait_for_equal)
    return error_code.value


def wait_for_bool_value(path, value, wait_for_equal):
    path = helper.fix_string_encoding(path)
    f = _get_lib().nxLibWaitForBoolValue
    f.argtypes = [POINTER(c_int32), c_char_p, c_int32, c_int32]
    error_code = c_int32(0)
    f(byref(error_code), path, value, wait_for_equal)
    return error_code.value


def make_unique_item(path, item_name):
    path = helper.fix_string_encoding(path)
    item_name = helper.fix_string_encoding(item_name)
    f = _get_lib().nxLibMakeUniqueItem
    f.restype = c_char_p
    f.argtypes = [POINTER(c_int32), c_char_p, c_char_p]
    error_code = c_int32(0)
    new_path = f(byref(error_code), path, item_name)
    if new_path is not None:
        new_path = new_path.decode()
    return new_path, error_code.value


def translate_error_code(return_code):
    if return_code < 0 or return_code > len(NX_ERRORS):
        return ''

    return NX_ERRORS[return_code]


def get_debug_messages():
    return_code = c_int32()
    f = _get_lib().nxLibGetDebugMessages
    f.restype = c_char_p
    f.argtypes = [POINTER(c_int32)]
    result = f(byref(return_code))
    check_return_code(return_code)
    return result.decode()


def get_debug_buffer():
    raise NotImplementedError()


def initialize(wait_for_initial_camera_refresh=True):
    if _lib_is_remote():
        raise Exception("Library is a remote nxlib. Only normal nxlib instances can use initialize.")
    f = _get_lib().nxLibInitialize
    return_code = c_int32()
    f.argtypes = [POINTER(c_int32), c_int32]
    f(byref(return_code), wait_for_initial_camera_refresh)
    check_return_code(return_code.value)


def finalize():
    if _lib_is_remote():
        raise Exception("Library is a remote nxlib. Only normal nxlib instances can use finalize.")
    f = _get_lib().nxLibFinalize
    return_code = c_int32()
    f.argtypes = [POINTER(c_int32)]
    f(byref(return_code))
    check_return_code(return_code.value)


def open_tcp_port(port_number=0, opened_port=0):
    if _lib_is_remote():
        raise Exception("Library is a remote nxlib. Only normal nxlib instances are allowed to open tcp ports.")
    f = _get_lib().nxLibOpenTcpPort
    return_code = c_int32()
    f.argtypes = [POINTER(c_int32), c_int32, c_int32]
    f(byref(return_code), port_number, opened_port)
    check_return_code(return_code.value)
    if __nx_testing__:
        print(port_number)


def close_tcp_port():
    if _lib_is_remote():
        raise Exception("Library is a remote nxlib. Only normal nxlib instances are allowed to close tcp ports.")
    f = _get_lib().nxLibCloseTcpPort
    return_code = c_int32()
    f.argtypes = [POINTER(c_int32)]
    f(byref(return_code))
    check_return_code(return_code.value)


def connect(hostname, port):
    if not _lib_is_remote():
        raise Exception("Cannot use connect function from a not remote NxLib.")
    hostname = helper.fix_string_encoding(hostname)
    return_code = c_int32(0)
    f = _get_lib().nxLibConnect
    f.argtypes = [POINTER(c_int32), c_char_p, c_int32]
    f(byref(return_code), hostname, port)
    check_return_code(return_code.value)


def disconnect():
    if not _lib_is_remote():
        raise Exception("Cannot use disconnenct function from a not remote NxLib.")
    f = _get_lib().nxLibDisconnect
    return_code = c_int32(0)
    f.argtypes = [POINTER(c_int32)]
    f(byref(return_code))
    check_return_code(return_code.value)



import ensenso_nxlib.api as nxlib
from .exception import NxLibError, NxLibException

import numpy as np

__all__ = ["NxLibItem"]

NXLIB_ITEM_TYPE_INVALID = 0
NXLIB_ITEM_TYPE_NULL = 1
NXLIB_ITEM_TYPE_NUMBER = 2
NXLIB_ITEM_TYPE_STRING = 3
NXLIB_ITEM_TYPE_BOOL = 4
NXLIB_ITEM_TYPE_ARRAY = 5
NXLIB_ITEM_TYPE_OBJECT = 6

NXLIB_ITEM_INEXISTENT = 3
NXLIB_OPERATION_SUCCEEDED = 0
NXLIB_BAD_REQUEST = 11
NXLIB_ITEM_TYPE_NOT_COMPATIBLE = 13

NXLIB_ITEM_SEPARATOR = '/'
NXLIB_INDEX_ESCAPE_CHAR = '\\'


class NxLibItem:
    def __init__(self, path=None):
        if path is None:
            path = ""
        self.path = path
        pass

    def _check_return_code(self, error_code):
        if error_code != NXLIB_OPERATION_SUCCEEDED:
            raise NxLibException(self.path, error_code)

    def __getitem__(self, value):
        if type(value) is str:
            return NxLibItem(self.path + NXLIB_ITEM_SEPARATOR + value)
        elif type(value) is int:
            return NxLibItem(self.path + NXLIB_ITEM_SEPARATOR + NXLIB_INDEX_ESCAPE_CHAR + str(value))
        else:
            raise NxLibError("Value cannot be added to NxLib path")

    def __setitem__(self, path, value):
        self[path].set_t(value)

    def compare(self, value):
        item_value = self.as_t()
        if type(item_value) == type(value):
            if item_value == value:
                return 0
            elif item_value < value:
                return -1
            else:
                return 1
        else:
            raise NxLibException(self.path, NXLIB_ITEM_TYPE_NOT_COMPATIBLE)

    def __lt__(self, value):
        return self.compare(value) < 0

    def __le__(self, value):
        return self.compare(value) <= 0

    def __eq__(self, value):
        return self.compare(value) == 0

    def __ne__(self, value):
        return self.compare(value) != 0

    def __gt__(self, value):
        return self.compare(value) > 0

    def __ge__(self, value):
        return self.compare(value) >= 0

    def __lshift__(self, other):
        if type(other) is str:
            self.set_json(str, True)
        elif isinstance(other, NxLibItem):
            self.set_json(other.as_json(), True)
        else:
            raise NxLibException(self.path, NXLIB_ITEM_TYPE_NOT_COMPATIBLE)

    def set_t(self, value):
        if value is None:
            self.set_null()
        elif type(value) is int:
            if (value > 2147483647 or value < -2147483648):
                self.set_double(value)
            else:
                self.set_int(value)
        elif type(value) is str:
            self.set_string(value)
        elif type(value) is bool:
            self.set_bool(value)
        # TODO: double is missing?
        else:
            raise NxLibException(self.path, NXLIB_ITEM_TYPE_NOT_COMPATIBLE)

    def set_null(self):
        error_code = nxlib.set_null(self.path)
        self._check_return_code(error_code)

    def set_double(self, value):
        error_code = nxlib.set_double(self.path, value)
        self._check_return_code(error_code)

    def set_int(self, value):
        error_code = nxlib.set_int(self.path, value)
        self._check_return_code(error_code)

    def set_bool(self, value):
        error_code = nxlib.set_bool(self.path, value)
        self._check_return_code(error_code)

    def set_string(self, value):
        error_code = nxlib.set_string(self.path, value)
        self._check_return_code(error_code)

    def set_json(self, value, only_writable_nodes=False):
        error_code = nxlib.set_json(self.path, value, only_writable_nodes)
        self._check_return_code(error_code)

    def set_binary_data(self, buffer, buffer_size_or_width=0, height=0,
                        channel_count=0, bytes_per_element=0, is_float=0):
        if buffer_size_or_width == 0:
            self.set_binary_data_from_cv(buffer)
        else:

            if (channel_count > 0):
                width = buffer_size_or_width
                error_code = nxlib.set_binary_formatted(self.path, buffer, width, height,
                                                        channel_count, bytes_per_element,
                                                        is_float)
            else:  # not formatted
                buffer_size = buffer_size_or_width
                error_code = nxlib.set_binary(self.path, buffer, buffer_size)
            self._check_return_code(error_code)

    def set_binary_data_from_cv(self, mat):
        if type(mat).__name__ != 'ndarray':
            raise NxLibException(self.path, NXLIB_ITEM_TYPE_NOT_COMPATIBLE)

        channel_count = mat.shape[2]
        is_float = False
        if mat.dtype == 'uint8' or mat.dtype == 'int8':
            bpe = 1
        elif mat.dtype == 'uint16' or mat.dtype == 'int16':
            bpe = 2
        elif mat.dtype == 'int32':
            bpe = 4
        elif mat.dtype == 'float32':
            bpe = 4
            is_float = True
        elif mat.dtype == 'float64':
            bpe = 8
            is_float = True

        buffer = np.ctypeslib.as_ctypes(mat)
        error_code = nxlib.set_binary_formatted(
            self.path, buffer, mat.shape[1], mat.shape[0], channel_count, bpe, is_float)
        self._check_return_code(error_code)

    def get_binary_data(self, buffer_size=0):
        if buffer_size == 0:
            buffer = self.create_buffer()
            buffer_size = buffer.shape[0] * buffer.shape[1] * buffer.shape[2] * buffer.dtype.itemsize
            cbuffer = np.ctypeslib.as_ctypes(buffer)
        else:
            cbuffer = []  # ?? to test # TODO
        _, _, error_code = nxlib.get_binary(
            self.path, cbuffer, buffer_size)
        self._check_return_code(error_code)
        return buffer

    def create_buffer(self):
        width, height, channel_count, bpe, is_float, _, _ = self.get_binary_data_info()
        nptype = np.uint8
        if is_float:
            if bpe == 4:
                nptype = np.float32
            elif bpe == 8:
                nptype = np.float64
        else:
            if bpe == 1:
                nptype = np.uint8
            elif bpe == 2:
                nptype = np.int16
            elif bpe == 4:
                nptype = np.int32

        image_buffer = np.zeros(
            (height, width, channel_count), nptype, order='C')

        return image_buffer

    def get_binary_data_info(self):
        width, height, channel_count, bpe, is_float, timestamp, error_code = nxlib.get_binary_info(
            self.path)
        self._check_return_code(error_code)
        return width, height, channel_count, bpe, is_float, timestamp, error_code

    def as_t(self):
        if self.is_null():
            return None
        elif self.is_number():
            return self.as_double()
        elif self.is_string():
            return self.as_string()
        elif self.is_bool():
            return self.as_bool()
        else:
            raise NxLibException(self.path, NXLIB_ITEM_TYPE_NOT_COMPATIBLE)

    def as_int(self):
        i, error_code = nxlib.get_int(self.path)
        self._check_return_code(error_code)
        return i

    def as_bool(self):
        b, error_code = nxlib.get_bool(self.path)
        self._check_return_code(error_code)
        return b

    def as_double(self):
        d, error_code = nxlib.get_double(self.path)
        self._check_return_code(error_code)
        return d

    def as_string(self):
        s, error_code = nxlib.get_string(self.path)
        self._check_return_code(error_code)
        return s

    def count(self):
        c, error_code = nxlib.get_count(self.path)
        self._check_return_code(error_code)
        return c

    def as_json(self, pretty_print=1, number_precision=2, scientific_number_format=0):
        json, error_code = nxlib.get_json(
            self.path, pretty_print, number_precision, scientific_number_format)
        self._check_return_code(error_code)
        return json

    def as_json_meta(self, num_levels=1, pretty_print=1, number_precision=2, scientific_number_format=0):
        json_meta, error_code = nxlib.get_json_meta(self.path, num_levels, pretty_print, number_precision,
                                                    scientific_number_format)
        self._check_return_code(error_code)
        return json_meta

    def is_null(self):
        t, error_code = nxlib.get_type(self.path)
        self._check_return_code(error_code)
        return t == NXLIB_ITEM_TYPE_NULL

    def is_string(self):
        t, error_code = nxlib.get_type(self.path)
        self._check_return_code(error_code)
        return t == NXLIB_ITEM_TYPE_STRING

    def is_number(self):
        t, error_code = nxlib.get_type(self.path)
        self._check_return_code(error_code)
        return t == NXLIB_ITEM_TYPE_NUMBER

    def is_bool(self):
        t, error_code = nxlib.get_type(self.path)
        self._check_return_code(error_code)
        return t == NXLIB_ITEM_TYPE_BOOL

    def is_array(self):
        t, error_code = nxlib.get_type(self.path)
        self._check_return_code(error_code)
        return t == NXLIB_ITEM_TYPE_ARRAY

    def is_object(self):
        t, error_code = nxlib.get_type(self.path)
        self._check_return_code(error_code)
        return t == NXLIB_ITEM_TYPE_OBJECT

    def type(self):  # overrides python type keyword ... !
        t, error_code = nxlib.get_type(self.path)
        self._check_return_code(error_code)
        return t

    def exists(self):
        t, error_code = nxlib.get_type(self.path)
        if error_code == NXLIB_ITEM_INEXISTENT:
            return False
        elif error_code == NXLIB_OPERATION_SUCCEEDED:
            return t != NXLIB_ITEM_TYPE_INVALID
        else:
            self._check_return_code(error_code)
            return False

    def name(self):
        item_name, error_code = nxlib.get_name(self.path)
        self._check_return_code(error_code)
        return item_name

    def erase(self):
        error_code = nxlib.erase(self.path)
        if error_code == NXLIB_ITEM_INEXISTENT:
            return
        self._check_return_code(error_code)

    def wait_for_change(self):
        error_code = nxlib.wait_for_change(self.path)
        self._check_return_code(error_code)

    def wait_for_type(self, awaited_type, wait_for_equal):
        error_code = nxlib.wait_for_type(self.path, awaited_type, wait_for_equal)
        self._check_return_code(error_code)

    def wait_for_value(self, value, wait_for_equal):
        if type(value) is int:
            error_code = nxlib.wait_for_int_value(self.path, value, wait_for_equal)
        elif type(value) is str:
            error_code = nxlib.wait_for_string_value(
                self.path, value, wait_for_equal)
        elif type(value) is bool:
            error_code = nxlib.wait_for_bool_value(self.path, value, wait_for_equal)
        self._check_return_code(error_code)

    def make_unique_item(self, item_name=None):
        new_path, error_code = nxlib.make_unique_item(self.path, item_name)
        self._check_return_code(error_code)
        if new_path:
            return NxLibItem(new_path)
        return NxLibItem()



import json

from .exception import NxLibError as _NxLibError
from .command import NxLibCommand as _NxLibCommand
from .helper import convert_camel_to_upper_snake as _convert_camel_to_upper_snake
from .helper import fix_nxlib_prefix as _fix_nxlib_prefix

NXLIB_ITEM_SEPARATOR = '/'
NXLIB_INDEX_ESCAPE_CHAR = '\\'
NXLIB_ITEM_FORBIDDEN_CHARS = "\r\n\"/\\\0"

NXLIB_BAD_REQUEST = 11
NXLIB_BUFFER_NOT_DIVISIBLE_BY_ELEMENT_SIZE = 16
NXLIB_BUFFER_TOO_SMALL = 15
NXLIB_CANNOT_CREATE_ITEM = 1
NXLIB_COMMAND_RUNNING = 19
NXLIB_CONNECTION_NOT_COMPATIBLE = 22
NXLIB_COULD_NOT_INTERPRET_JSON_TEXT = 2
NXLIB_COULD_NOT_LOAD_FUNCTION = 21
NXLIB_COULD_NOT_LOAD_LIBRARY = 20
NXLIB_COULD_NOT_OPEN_PORT = 6
NXLIB_DEBUG_MESSAGE_OVERFLOW = 18
NXLIB_EXECUTION_FAILED = 17
NXLIB_INITIALIZATION_NOT_ALLOWED = 23
NXLIB_INTERNAL_ERROR = 7
NXLIB_INVALID_BUFFER_SIZE = 14
NXLIB_ITEM_INEXISTENT = 3
NXLIB_ITEM_TYPE_NOT_COMPATIBLE = 13
NXLIB_METHOD_INVALID = 10
NXLIB_NESTING_LIMIT_REACHED = 24
NXLIB_NO_DEBUG_DATA = 5
NXLIB_NO_OPEN_PROFILE_BLOCK = 25
NXLIB_NOT_CONNECTED = 9
NXLIB_OPERATION_SUCCEEDED = 0
NXLIB_TIMEOUT = 8
CMD_ADAPTER_CONFIGURATION = "AdapterConfiguration"
CMD_ADD_PATTERN_BUFFER_VIEW = "AddPatternBufferView"
CMD_ADJUST_EXPOSURE_AND_GAIN = "AdjustExposureAndGain"
CMD_BREAK = "Break"
CMD_CALIBRATE = "Calibrate"
CMD_CALIBRATE_HAND_EYE = "CalibrateHandEye"
CMD_CALIBRATE_IN_BACKGROUND = "CalibrateInBackground"
CMD_CALIBRATE_PATTERN = "CalibratePattern"
CMD_CALIBRATE_UPDATE_PATTERNS = "CalibrateUpdatePatterns"
CMD_CALIBRATE_WORKSPACE = "CalibrateWorkspace"
CMD_CAPTURE = "Capture"
CMD_CHAIN_TRANSFORMATIONS = "ChainTransformations"
CMD_CLEAR_IMAGE_BUFFER = "ClearImageBuffer"
CMD_CLEAR_IMAGES = "ClearImages"
CMD_CLEAR_OVERLAY = "ClearOverlay"
CMD_CLOSE = "Close"
CMD_COLLECT_PATTERN = "CollectPattern"
CMD_COLLECT_PLANE_POINTS = "CollectPlanePoints"
CMD_COMPUTE_DISPARITY_MAP = "ComputeDisparityMap"
CMD_COMPUTE_IMAGE_CONTRAST = "ComputeImageContrast"
CMD_COMPUTE_NORMALS = "ComputeNormals"
CMD_COMPUTE_POINT_MAP = "ComputePointMap"
CMD_CONVERT_TRANSFORMATION = "ConvertTransformation"
CMD_CREATE_CAMERA = "CreateCamera"
CMD_DELETE_CAMERA = "DeleteCamera"
CMD_DISCARD_PATTERNS = "DiscardPatterns"
CMD_ESTIMATE_DISPARITY_SETTINGS = "EstimateDisparitySettings"
CMD_ESTIMATE_PATTERN_POSE = "EstimatePatternPose"
CMD_ETHERNET_CONFIGURATION = "EthernetConfiguration"
CMD_FILTER_PATTERN_BUFFER = "FilterPatternBuffer"
CMD_FIT_PRIMITIVE = "FitPrimitive"
CMD_FLASH = "Flash"
CMD_GENERATE_CALIBRATION_PATTERN = "GenerateCalibrationPattern"
CMD_GET_CONSTANTS = "GetConstants"
CMD_GET_MODEL_INFO = "GetModelInfo"
CMD_GET_PATTERN_BUFFER = "GetPatternBuffer"
CMD_GET_PATTERN_BUFFERS = "GetPatternBuffers"
CMD_GET_RAW_CALIBRATION_DATA = "GetRawCalibrationData"
CMD_INTERNAL_PWM = "InternalPwm"
CMD_LOAD_CALIBRATION = "LoadCalibration"
CMD_LOAD_IMAGE = "LoadImage"
CMD_LOAD_TEXT = "LoadText"
CMD_LOAD_U_EYE_PARAMETER_SET = "LoadUEyeParameterSet"
CMD_MEASURE_CALIBRATION = "MeasureCalibration"
CMD_NETWORK_CONTROL = "NetworkControl"
CMD_OPEN = "Open"
CMD_PATTERN_BUFFER_INFO = "PatternBufferInfo"
CMD_PROJECT_PATTERN = "ProjectPattern"
CMD_RECALIBRATE = "Recalibrate"
CMD_RECTIFY_IMAGES = "RectifyImages"
CMD_REDUCE_PATTERN_BUFFER = "ReducePatternBuffer"
CMD_REDUCE_PATTERNS = "ReducePatterns"
CMD_REMOVE_PATTERN_BUFFER_VIEW = "RemovePatternBufferView"
CMD_RENDER_DEPTH_MAP = "RenderDepthMap"
CMD_RENDER_POINT_MAP = "RenderPointMap"
CMD_RENDER_VIEW = "RenderView"
CMD_RETRIEVE = "Retrieve"
CMD_SAVE_IMAGE = "SaveImage"
CMD_SAVE_MODEL = "SaveModel"
CMD_SAVE_TEXT = "SaveText"
CMD_SET_PATTERN_BUFFER = "SetPatternBuffer"
CMD_SET_PATTERN_BUFFERS = "SetPatternBuffers"
CMD_SET_STATUS_LEDS = "SetStatusLeds"
CMD_SIMULATE_PHYSICS = "SimulatePhysics"
CMD_STORE_CALIBRATION = "StoreCalibration"
CMD_SYNCHRONIZE = "Synchronize"
CMD_TRIGGER = "Trigger"
CMD_UPDATE_FIRMWARE = "UpdateFirmware"
CMD_VISUALIZE_PATTERN_BUFFER = "VisualizePatternBuffer"
ERR_CALIBRATION_FAILED = "CalibrationFailed"
ERR_CAMERA_NOT_FOUND = "CameraNotFound"
ERR_CHANGED_MODEL = "ChangedModel"
ERR_COMMAND_NOT_ALLOWED = "CommandNotAllowed"
ERR_COMMAND_UNKNOWN = "CommandUnknown"
ERR_EMPTY_IMAGE = "EmptyImage"
ERR_INVALID_CALIBRATION_DATA = "InvalidCalibrationData"
ERR_INVALID_PAIRING_DATA = "InvalidPairingData"
ERR_INVALID_PATTERN_BUFFER = "InvalidPatternBuffer"
ERR_NO_WORKSPACE_LINK = "NoWorkspaceLink"
ERR_NOT_ENOUGH_POINTS_FOR_PRIMITIVE = "NotEnoughPointsForPrimitive"
ERR_OPERATION_CANCELED = "OperationCanceled"
ERR_PATTERN_BUFFER_LOCKED = "PatternBufferLocked"
ERR_PATTERN_DATA_INCOMPATIBLE = "PatternDataIncompatible"
ERR_PATTERN_NOT_DECODABLE = "PatternNotDecodable"
ERR_PATTERN_NOT_FOUND = "PatternNotFound"
ERR_SENSOR_NOT_COMPATIBLE = "SensorNotCompatible"
ERR_UNHANDLED_EXCEPTION = "UnhandledException"
ERR_WIRING_TEST_FAILED = "WiringTestFailed"
NXLIB_ITEM_TYPE_ARRAY = 5
NXLIB_ITEM_TYPE_BOOL = 4
NXLIB_ITEM_TYPE_INVALID = 0
NXLIB_ITEM_TYPE_NULL = 1
NXLIB_ITEM_TYPE_NUMBER = 2
NXLIB_ITEM_TYPE_OBJECT = 6
NXLIB_ITEM_TYPE_STRING = 3
ITM_ABSOLUTE_BLACK_LEVEL_OFFSET = "AbsoluteBlackLevelOffset"
ITM_ACTIVE = "Active"
ITM_ADAPTERS = "Adapters"
ITM_ADJUST_EXPOSURE = "AdjustExposure"
ITM_ADJUST_GAIN = "AdjustGain"
ITM_ALIGN_AXIS = "AlignAxis"
ITM_ALL = "All"
ITM_ALLOW_FIRMWARE_UPLOAD = "AllowFirmwareUpload"
ITM_AMBIENT = "Ambient"
ITM_AND = "And"
ITM_ANGLE = "Angle"
ITM_ANGLES = "Angles"
ITM_APERTURE = "Aperture"
ITM_API_ERRORS = "ApiErrors"
ITM_APPLICATION = "Application"
ITM_APPLY = "Apply"
ITM_ARCHITECTURE = "Architecture"
ITM_AREA_OF_INTEREST = "AreaOfInterest"
ITM_ASSEMBLY_CALIBRATION = "AssemblyCalibration"
ITM_ASSEMBLY_PATTERN = "AssemblyPattern"
ITM_ASYNCHRONOUSLY_TRIGGERED = "AsynchronouslyTriggered"
ITM_AUTO_BLACK_LEVEL = "AutoBlackLevel"
ITM_AUTO_EXPOSURE = "AutoExposure"
ITM_AUTO_FOCUS = "AutoFocus"
ITM_AUTO_GAIN = "AutoGain"
ITM_AUTO_SWAP = "AutoSwap"
ITM_AUTOMATIC = "Automatic"
ITM_AVAILABLE = "Available"
ITM_AVERAGE = "Average"
ITM_AVERAGE_POSE_ERROR = "AveragePoseError"
ITM_AXIS = "Axis"
ITM_BACKGROUND = "Background"
ITM_BANDWIDTH = "Bandwidth"
ITM_BANDWIDTH_LIMIT = "BandwidthLimit"
ITM_BASELINE = "Baseline"
ITM_BINARY_INFO = "BinaryInfo"
ITM_BINNING = "Binning"
ITM_BINNING_SHIFT = "BinningShift"
ITM_BLACK_LEVEL_OFFSET = "BlackLevelOffset"
ITM_BLACK_LEVEL_OFFSET_CALIBRATION = "BlackLevelOffsetCalibration"
ITM_BLIND = "Blind"
ITM_BLUR = "Blur"
ITM_BLUR_MIN_MAX = "BlurMinMax"
ITM_BOARD = "Board"
ITM_BOOTLOADER = "Bootloader"
ITM_BORDER_SPREAD = "BorderSpread"
ITM_BOUNDING_BOX = "BoundingBox"
ITM_BRIGHTNESS = "Brightness"
ITM_BUFFER = "Buffer"
ITM_BUFFER_COUNT = "BufferCount"
ITM_BUFFER_SIZE = "BufferSize"
ITM_BUILD = "Build"
ITM_BY_EEPROM_ID = "ByEepromId"
ITM_BY_SERIAL_NO = "BySerialNo"
ITM_CPU = "CPU"
ITM_CRC_ERROR_COUNT = "CRCErrorCount"
ITM_CUDA = "CUDA"
ITM_CAL_TAB_TYPE = "CalTabType"
ITM_CAL_TABS = "CalTabs"
ITM_CALCULATE_IP = "CalculateIp"
ITM_CALIBRATED = "Calibrated"
ITM_CALIBRATION = "Calibration"
ITM_CALIBRATION_FILE = "CalibrationFile"
ITM_CALIBRATION_HISTORY = "CalibrationHistory"
ITM_CAMERA = "Camera"
ITM_CAMERAS = "Cameras"
ITM_CAPTURE = "Capture"
ITM_CAPTURE_EVENTS = "CaptureEvents"
ITM_CENTER = "Center"
ITM_CHECK = "Check"
ITM_CHECK_DAEMON = "CheckDaemon"
ITM_CLEAR_IMAGE_BUFFER_ON_TRIGGER = "ClearImageBufferOnTrigger"
ITM_CLOCK_RATE = "ClockRate"
ITM_CODE_METER = "CodeMeter"
ITM_COLOR = "Color"
ITM_COLOR_OFFSET = "ColorOffset"
ITM_COLOR_REPETITION_DISTANCE = "ColorRepetitionDistance"
ITM_COMMAND = "Command"
ITM_COMMANDS = "Commands"
ITM_COMMON_ITEMS = "CommonItems"
ITM_COMMON_VALUES = "CommonValues"
ITM_COMPONENT_THRESHOLD = "ComponentThreshold"
ITM_COMPUTE_CAPABILITY = "ComputeCapability"
ITM_COMPUTE_DISPARITY_MAP = "ComputeDisparityMap"
ITM_CONFIGURATION = "Configuration"
ITM_CONNECTED = "Connected"
ITM_CONNECTED_CAMERAS = "ConnectedCameras"
ITM_CONNECTED_PATTERNS = "ConnectedPatterns"
ITM_CONTENT = "Content"
ITM_CONTRAST = "Contrast"
ITM_CORES = "Cores"
ITM_COST_SCALE = "CostScale"
ITM_COUNT = "Count"
ITM_DHCP = "DHCP"
ITM_DAY = "Day"
ITM_DEBUG = "Debug"
ITM_DECODE_DATA = "DecodeData"
ITM_DEFAULT = "Default"
ITM_DEFAULT_GATEWAY = "DefaultGateway"
ITM_DEFAULT_PARAMETERS = "DefaultParameters"
ITM_DEFINED_POSE = "DefinedPose"
ITM_DEPTH = "Depth"
ITM_DEPTH_CHANGE_COST = "DepthChangeCost"
ITM_DEPTH_STEP_COST = "DepthStepCost"
ITM_DESTINATION = "Destination"
ITM_DEVICE = "Device"
ITM_DEVICES = "Devices"
ITM_DIFFUSE = "Diffuse"
ITM_DIRECTION = "Direction"
ITM_DISPARITY = "Disparity"
ITM_DISPARITY_ACCURACY = "DisparityAccuracy"
ITM_DISPARITY_MAP = "DisparityMap"
ITM_DISPARITY_MAP_OFFSET = "DisparityMapOffset"
ITM_DISPARITY_STEP = "DisparityStep"
ITM_DISTANCE = "Distance"
ITM_DISTORTION = "Distortion"
ITM_DOWNLOAD_IMAGES = "DownloadImages"
ITM_DOWNSAMPLE = "Downsample"
ITM_DRAW_AXES = "DrawAxes"
ITM_DRAW_ONLY = "DrawOnly"
ITM_DRAW_ORIGIN = "DrawOrigin"
ITM_DRIVER = "Driver"
ITM_DURATION = "Duration"
ITM_DUTY_CYCLE = "DutyCycle"
ITM_DYNAMIC = "Dynamic"
ITM_DYNAMIC_CALIBRATION_LIMIT_REACHED = "DynamicCalibrationLimitReached"
ITM_DYNAMIC_OFFSETS = "DynamicOffsets"
ITM_EEPROM_FORMAT = "EepromFormat"
ITM_EEPROM_ID = "EepromId"
ITM_ENABLE_IP_FILTER = "EnableIpFilter"
ITM_ENABLED = "Enabled"
ITM_ENCODING = "Encoding"
ITM_EPIPOLAR = "Epipolar"
ITM_EPIPOLAR_ERROR = "EpipolarError"
ITM_ERROR_SYMBOL = "ErrorSymbol"
ITM_ERROR_TEXT = "ErrorText"
ITM_ERRORS = "Errors"
ITM_ESTIMATE_GRID_SPACING = "EstimateGridSpacing"
ITM_ETHERNET = "Ethernet"
ITM_ETHERNET_CONFIG_MODE = "EthernetConfigMode"
ITM_EXECUTE = "Execute"
ITM_EXPOSURE = "Exposure"
ITM_EXT = "Ext"
ITM_EXTENDED_TYPE = "ExtendedType"
ITM_FACTORY = "Factory"
ITM_FAILURE_PROBABILITY = "FailureProbability"
ITM_FAR = "Far"
ITM_FILE_OUTPUT = "FileOutput"
ITM_FILE_PREFIX = "FilePrefix"
ITM_FILENAME = "Filename"
ITM_FILL_XY_COORDINATES = "FillXYCoordinates"
ITM_FILLING = "Filling"
ITM_FILTER = "Filter"
ITM_FILTERS = "Filters"
ITM_FINAL_TRIGGER = "FinalTrigger"
ITM_FIRMWARE_UPLOAD = "FirmwareUpload"
ITM_FIRMWARE_VERSION = "FirmwareVersion"
ITM_FIXED = "Fixed"
ITM_FLAGS = "Flags"
ITM_FLASH_DELAY = "FlashDelay"
ITM_FLEX_VIEW = "FlexView"
ITM_FOCAL_LENGTH = "FocalLength"
ITM_FOCUS = "Focus"
ITM_FOCUS_DISTANCE = "FocusDistance"
ITM_FOLDER_PATH = "FolderPath"
ITM_FOLLOW_DYNAMIC_OFFSETS = "FollowDynamicOffsets"
ITM_FONT = "Font"
ITM_FORCE = "Force"
ITM_FORCE_GRAYSCALE = "ForceGrayscale"
ITM_FORCED_RAW_IMAGE_SIZE = "ForcedRawImageSize"
ITM_FORCED_RECTIFIED_IMAGE_SIZE = "ForcedRectifiedImageSize"
ITM_FREQUENCY = "Frequency"
ITM_FROM_DYNAMIC = "FromDynamic"
ITM_FRONT_LIGHT = "FrontLight"
ITM_GAIN = "Gain"
ITM_GAIN_BOOST = "GainBoost"
ITM_GATEWAY = "Gateway"
ITM_GENERATE_EMPTY_CALIBRATION = "GenerateEmptyCalibration"
ITM_GLOBAL_PATTERN_DATA_UPDATED = "GlobalPatternDataUpdated"
ITM_GLOBAL_SHUTTER = "GlobalShutter"
ITM_GLOW = "Glow"
ITM_GRAVITY = "Gravity"
ITM_GREEN = "Green"
ITM_GRID_SIZE = "GridSize"
ITM_GRID_SPACING = "GridSpacing"
ITM_GROUND_PLANE = "GroundPlane"
ITM_HUD = "HUD"
ITM_HALF_VERGENCE = "HalfVergence"
ITM_HARDWARE_FAILURE = "HardwareFailure"
ITM_HARDWARE_GAMMA = "HardwareGamma"
ITM_HAS_LICENSE = "HasLicense"
ITM_HASH = "Hash"
ITM_HDR = "Hdr"
ITM_HEIGHT = "Height"
ITM_HIGH_QUALITY_RENDERING = "HighQualityRendering"
ITM_HORIZONTAL = "Horizontal"
ITM_IO = "IO"
ITM_IGNORE_ENSENSO_PATTERN_ENCODING = "IgnoreEnsensoPatternEncoding"
ITM_IMAGE_BUFFER = "ImageBuffer"
ITM_IMAGE_DOWNLOAD_LIMIT = "ImageDownloadLimit"
ITM_IMAGE_FOLDER = "ImageFolder"
ITM_IMAGE_NAME = "ImageName"
ITM_IMAGE_OFFSET = "ImageOffset"
ITM_IMAGE_SET = "ImageSet"
ITM_IMAGE_SIZE = "ImageSize"
ITM_IMAGES = "Images"
ITM_INDEX = "Index"
ITM_INFO = "Info"
ITM_INFO_TIMEOUT = "InfoTimeout"
ITM_INITIAL_TRIGGER = "InitialTrigger"
ITM_INITIAL_VELOCITY = "InitialVelocity"
ITM_INLIER_COUNT = "InlierCount"
ITM_INLIER_FRACTION = "InlierFraction"
ITM_INLIER_THRESHOLD = "InlierThreshold"
ITM_INPUT = "Input"
ITM_INTEGRATED = "Integrated"
ITM_INTERFACE = "Interface"
ITM_INTERNAL_TRIGGER = "InternalTrigger"
ITM_INTERVAL = "Interval"
ITM_INVERSE = "Inverse"
ITM_INVERT = "Invert"
ITM_IP_ADDRESS = "IpAddress"
ITM_IP_BROADCAST = "IpBroadcast"
ITM_IP_SUBNET_MASK = "IpSubnetMask"
ITM_ITEM_TYPES = "ItemTypes"
ITM_ITEMS = "Items"
ITM_ITERATIONS = "Iterations"
ITM_K1 = "K1"
ITM_K2 = "K2"
ITM_K3 = "K3"
ITM_KEEP_ALIVE_TIMEOUT = "KeepAliveTimeout"
ITM_LED = "LED"
ITM_LATEST_MESSAGE = "LatestMessage"
ITM_LEFT = "Left"
ITM_LEFT_BOTTOM = "LeftBottom"
ITM_LEFT_TOP = "LeftTop"
ITM_LENS = "Lens"
ITM_LENSES = "Lenses"
ITM_LEVEL = "Level"
ITM_LIGHTING = "Lighting"
ITM_LINK = "Link"
ITM_LINKS = "Links"
ITM_LIST_CAL_TABS = "ListCalTabs"
ITM_LIST_LENSES = "ListLenses"
ITM_LIST_MODELS = "ListModels"
ITM_LIST_PATTERNS = "ListPatterns"
ITM_LIST_SENSORS = "ListSensors"
ITM_LOAD_CALIBRATION = "LoadCalibration"
ITM_LOCAL_EEPROM_FILE = "LocalEepromFile"
ITM_LOW_BANDWIDTH = "LowBandwidth"
ITM_MAC = "MAC"
ITM_MAC_ADDRESSES = "MacAddresses"
ITM_MAJOR = "Major"
ITM_MARK_FILTER_REGIONS = "MarkFilterRegions"
ITM_MARK_ONLY = "MarkOnly"
ITM_MASK = "Mask"
ITM_MASS = "Mass"
ITM_MASTER = "Master"
ITM_MATERIAL_BLUR = "MaterialBlur"
ITM_MAX = "Max"
ITM_MAX_EEPROM_FORMAT = "MaxEepromFormat"
ITM_MAX_FILE_SIZE = "MaxFileSize"
ITM_MAX_FLASH_TIME = "MaxFlashTime"
ITM_MAX_GAIN = "MaxGain"
ITM_MAX_POSE_ERROR = "MaxPoseError"
ITM_MAX_TOTAL_SIZE = "MaxTotalSize"
ITM_MAXIMUM_TRANSMISSION_UNIT = "MaximumTransmissionUnit"
ITM_MEASURE_CALIBRATION = "MeasureCalibration"
ITM_MEASURE_CONTRAST = "MeasureContrast"
ITM_MEASUREMENT_VOLUME = "MeasurementVolume"
ITM_MEDIAN_FILTER_RADIUS = "MedianFilterRadius"
ITM_MEMORY = "Memory"
ITM_MESSAGE = "Message"
ITM_MESSAGES = "Messages"
ITM_META_DATA = "MetaData"
ITM_META_INFO = "MetaInfo"
ITM_METHOD = "Method"
ITM_MIN = "Min"
ITM_MIN_DISPARITY = "MinDisparity"
ITM_MINIMUM_DISPARITY = "MinimumDisparity"
ITM_MINIMUM_VOLTAGE = "MinimumVoltage"
ITM_MINOR = "Minor"
ITM_MIRROR = "Mirror"
ITM_MODE = "Mode"
ITM_MODEL_NAME = "ModelName"
ITM_MODELS = "Models"
ITM_MONO_INTRINSIC = "MonoIntrinsic"
ITM_MONOCULAR = "Monocular"
ITM_MONOCULAR_CALIBRATION = "MonocularCalibration"
ITM_MONOCULAR_PATTERN_COUNT = "MonocularPatternCount"
ITM_MONTH = "Month"
ITM_MULTI_EXPOSURE_FACTOR = "MultiExposureFactor"
ITM_NAME = "Name"
ITM_NEAR = "Near"
ITM_NETWORK_ADAPTER = "NetworkAdapter"
ITM_NETWORK_ADAPTER_IP_ADDRESS = "NetworkAdapterIpAddress"
ITM_NETWORK_ADAPTER_IP_SUBNET_MASK = "NetworkAdapterIpSubnetMask"
ITM_NETWORK_TYPE = "NetworkType"
ITM_NODE = "Node"
ITM_NODES = "Nodes"
ITM_NOISE_LEVEL = "NoiseLevel"
ITM_NORMAL = "Normal"
ITM_NORMALS = "Normals"
ITM_NUMBER_OF_DISPARITIES = "NumberOfDisparities"
ITM_NUMBER_OF_IMAGE_SETS = "NumberOfImageSets"
ITM_NX_LIB = "NxLib"
ITM_OBJECT_POINTS = "ObjectPoints"
ITM_OBJECTS = "Objects"
ITM_OFFSET = "Offset"
ITM_OPEN = "Open"
ITM_OPEN_MP = "OpenMP"
ITM_OPERATING_SYSTEM = "OperatingSystem"
ITM_OPERATION = "Operation"
ITM_OPTICAL_AXIS = "OpticalAxis"
ITM_OPTIMIZATION_PROFILE = "OptimizationProfile"
ITM_OPTIONS = "Options"
ITM_OR = "Or"
ITM_OUTER_SIZE = "OuterSize"
ITM_OUTPUT = "Output"
ITM_OVERFLOW_POLICY = "OverflowPolicy"
ITM_OVERLAY = "Overlay"
ITM_OVERTEMPERATURE = "Overtemperature"
ITM_OVERWRITE_WITH_DYNAMIC = "OverwriteWithDynamic"
ITM_PACKETS_RESENT = "PacketsResent"
ITM_PADDING = "Padding"
ITM_PAIRED = "Paired"
ITM_PARAMETERS = "Parameters"
ITM_PATTERN = "Pattern"
ITM_PATTERN_BUFFER = "PatternBuffer"
ITM_PATTERN_COUNT = "PatternCount"
ITM_PATTERN_POSE = "PatternPose"
ITM_PATTERN_TYPE = "PatternType"
ITM_PATTERN_VOLUME = "PatternVolume"
ITM_PATTERNS = "Patterns"
ITM_PERSISTENT_OVERLAY = "PersistentOverlay"
ITM_PERSISTENT_PARAMETERS = "PersistentParameters"
ITM_PHASE_INTERPOLATION = "PhaseInterpolation"
ITM_PHYSICS = "Physics"
ITM_PIXEL_CLOCK = "PixelClock"
ITM_PIXEL_PITCH = "PixelPitch"
ITM_PIXEL_SIZE = "PixelSize"
ITM_PLANE = "Plane"
ITM_PLUGGED = "Plugged"
ITM_POINT_MAP = "PointMap"
ITM_POINTS = "Points"
ITM_POLARITY = "Polarity"
ITM_PORT = "Port"
ITM_POSE = "Pose"
ITM_POSE_ERROR = "PoseError"
ITM_POSES = "Poses"
ITM_POST_PROCESSING = "PostProcessing"
ITM_PRIMITIVE = "Primitive"
ITM_PROGRESS = "Progress"
ITM_PROGRESS_FACTOR = "ProgressFactor"
ITM_PROGRESS_GRID = "ProgressGrid"
ITM_PROGRESS_MASK = "ProgressMask"
ITM_PROGRESS_MASK_FACTOR = "ProgressMaskFactor"
ITM_PROGRESS_MODE = "ProgressMode"
ITM_PROJECTOR = "Projector"
ITM_PROJECTOR_DUTY_CYCLE = "ProjectorDutyCycle"
ITM_PROJECTOR_PATTERN = "ProjectorPattern"
ITM_PROJECTOR_POWER = "ProjectorPower"
ITM_PROPAGATION_DECAY = "PropagationDecay"
ITM_PROTECTION = "Protection"
ITM_RADIUS = "Radius"
ITM_RAW = "Raw"
ITM_RAW_AOI_INCREMENTS = "RawAoiIncrements"
ITM_RECALIBRATE = "Recalibrate"
ITM_RECTIFICATION = "Rectification"
ITM_RECTIFIED = "Rectified"
ITM_REDUCE = "Reduce"
ITM_REDUCED = "Reduced"
ITM_REFINEMENT = "Refinement"
ITM_REGION = "Region"
ITM_REGION_FILTER_DOWNSAMPLING = "RegionFilterDownsampling"
ITM_REGION_SIZE = "RegionSize"
ITM_RELATIVE = "Relative"
ITM_RELATIVE_AVERAGE_POSE_ERROR = "RelativeAveragePoseError"
ITM_RELATIVE_MAX_POSE_ERROR = "RelativeMaxPoseError"
ITM_REMOTE = "Remote"
ITM_RENDER_DEPTH_MAP = "RenderDepthMap"
ITM_RENDER_GROUND_TRUTH = "RenderGroundTruth"
ITM_RENDER_POINT_MAP = "RenderPointMap"
ITM_RENDER_POINT_MAP_TEXTURE = "RenderPointMapTexture"
ITM_RENDER_VIEW = "RenderView"
ITM_REPROJECTION = "Reprojection"
ITM_REPROJECTION_ERROR = "ReprojectionError"
ITM_REPROJECTION_ERROR_SCALE = "ReprojectionErrorScale"
ITM_RESET_CLOCK = "ResetClock"
ITM_RESIDUAL = "Residual"
ITM_RESTART_DAEMON = "RestartDaemon"
ITM_RESULT = "Result"
ITM_RETRIEVED = "Retrieved"
ITM_RETURN_ALL_PATTERN = "ReturnAllPattern"
ITM_REVERSE_PATH_FILTERING = "ReversePathFiltering"
ITM_RIGHT = "Right"
ITM_RIGHT_BOTTOM = "RightBottom"
ITM_RIGHT_TOP = "RightTop"
ITM_ROTATE = "Rotate"
ITM_ROTATION = "Rotation"
ITM_RUNNING = "Running"
ITM_RX = "Rx"
ITM_RY = "Ry"
ITM_SCALED_AREA_OF_INTEREST = "ScaledAreaOfInterest"
ITM_SCALED_MEASUREMENT_VOLUME = "ScaledMeasurementVolume"
ITM_SCALED_MINIMUM_DISPARITY = "ScaledMinimumDisparity"
ITM_SCALED_NUMBER_OF_DISPARITIES = "ScaledNumberOfDisparities"
ITM_SCALING = "Scaling"
ITM_SCORE = "Score"
ITM_SENSOR = "Sensor"
ITM_SENSORS = "Sensors"
ITM_SERIAL_NUMBER = "SerialNumber"
ITM_SETUP = "Setup"
ITM_SHADOWING_THRESHOLD = "ShadowingThreshold"
ITM_SHININESS = "Shininess"
ITM_SHOW_CAMERAS = "ShowCameras"
ITM_SHOW_GRID = "ShowGrid"
ITM_SHOW_OBJECT_POINTS = "ShowObjectPoints"
ITM_SHOW_PATTERN = "ShowPattern"
ITM_SHOW_PATTERN_POINTS = "ShowPatternPoints"
ITM_SHOW_PATTERNS = "ShowPatterns"
ITM_SHOW_RECTIFIED_AREA = "ShowRectifiedArea"
ITM_SHOW_SURFACE = "ShowSurface"
ITM_SHOW_USER_DEFINED_MODELS = "ShowUserDefinedModels"
ITM_SINK = "Sink"
ITM_SIZE = "Size"
ITM_SKEW = "Skew"
ITM_SOURCE = "Source"
ITM_SPECKLE_REMOVAL = "SpeckleRemoval"
ITM_SPECULAR = "Specular"
ITM_SPLIT_ROTATION = "SplitRotation"
ITM_START_DAEMON = "StartDaemon"
ITM_STATIC_BUFFER_COUNT = "StaticBufferCount"
ITM_STATIC_BUFFERS = "StaticBuffers"
ITM_STATUS = "Status"
ITM_STEREO = "Stereo"
ITM_STEREO_CALIBRATION = "StereoCalibration"
ITM_STEREO_CALIBRATION_ONLY = "StereoCalibrationOnly"
ITM_STEREO_EXTRINSIC = "StereoExtrinsic"
ITM_STEREO_INTRINSIC = "StereoIntrinsic"
ITM_STEREO_MATCHING = "StereoMatching"
ITM_STEREO_PATTERN_COUNT = "StereoPatternCount"
ITM_STOP_DAEMON = "StopDaemon"
ITM_SUBSAMPLING = "Subsampling"
ITM_SUBSET = "Subset"
ITM_SURFACE_CONNECTIVITY = "SurfaceConnectivity"
ITM_SYMBOL = "Symbol"
ITM_SYSTEM_INFO = "SystemInfo"
ITM_T1 = "T1"
ITM_T2 = "T2"
ITM_TARGET = "Target"
ITM_TARGET_BRIGHTNESS = "TargetBrightness"
ITM_TEMPERATURE = "Temperature"
ITM_TEMPORARY = "Temporary"
ITM_TEXT = "Text"
ITM_TEXTURE = "Texture"
ITM_THICKNESS = "Thickness"
ITM_THREADS = "Threads"
ITM_TILT_DIRECTION = "TiltDirection"
ITM_TIME = "Time"
ITM_TIME_EXECUTE = "TimeExecute"
ITM_TIME_FINALIZE = "TimeFinalize"
ITM_TIME_PREPARE = "TimePrepare"
ITM_TIMEOUT = "Timeout"
ITM_TIMESTAMP = "Timestamp"
ITM_TOLERANCE = "Tolerance"
ITM_TOP = "Top"
ITM_TRANSFORMATION = "Transformation"
ITM_TRANSFORMATIONS = "Transformations"
ITM_TRANSLATION = "Translation"
ITM_TRANSPORT_LAYER = "TransportLayer"
ITM_TRIGGER_DELAY = "TriggerDelay"
ITM_TRIGGER_MODE = "TriggerMode"
ITM_TRIGGERED = "Triggered"
ITM_TYPE = "Type"
ITM_U_EYE = "UEye"
ITM_USB = "USB"
ITM_UNIQUE_NAME = "UniqueName"
ITM_UNIQUENESS_OFFSET = "UniquenessOffset"
ITM_UNIQUENESS_RATIO = "UniquenessRatio"
ITM_UPDATE_GLOBAL_PATTERN_DATA = "UpdateGlobalPatternData"
ITM_UPDATER = "Updater"
ITM_URL = "Url"
ITM_USE_BUFFERED_PATTERNS = "UseBufferedPatterns"
ITM_USE_DISPARITY_MAP_AREA_OF_INTEREST = "UseDisparityMapAreaOfInterest"
ITM_USE_FLOAT16 = "UseFloat16"
ITM_USE_MODEL = "UseModel"
ITM_USE_OPEN_GL = "UseOpenGL"
ITM_USE_RECALIBRATOR = "UseRecalibrator"
ITM_USE_STEREO_TEXTURES = "UseStereoTextures"
ITM_USER = "User"
ITM_VALID_FIRMWARE = "ValidFirmware"
ITM_VALID_IP_ADDRESS = "ValidIpAddress"
ITM_VALID_REGION = "ValidRegion"
ITM_VALUE = "Value"
ITM_VALUES = "Values"
ITM_VERGENCE = "Vergence"
ITM_VERSION = "Version"
ITM_VERTICAL = "Vertical"
ITM_VIEW_POSE = "ViewPose"
ITM_VIGNETTING = "Vignetting"
ITM_WAIT_FOR = "WaitFor"
ITM_WAIT_FOR_PROJECTOR = "WaitForProjector"
ITM_WAIT_FOR_RECALIBRATION = "WaitForRecalibration"
ITM_WIDTH = "Width"
ITM_WIRING_TEST = "WiringTest"
ITM_WITH_OVERLAY = "WithOverlay"
ITM_WORLD_COORDINATES = "WorldCoordinates"
ITM_WRITE_FREQUENCY = "WriteFrequency"
ITM_YEAR = "Year"
ITM_YELLOW = "Yellow"
ITM_Z_BUFFER_ONLY = "ZBufferOnly"
VAL_ADD = "Add"
VAL_ALIGNED = "Aligned"
VAL_ALIGNED_AND_DIAGONAL = "AlignedAndDiagonal"
VAL_ALL = "All"
VAL_ALLOW = "Allow"
VAL_ARRAY = "Array"
VAL_ASSEMBLY = "Assembly"
VAL_AUTO = "Auto"
VAL_AVAILABLE = "Available"
VAL_AXIS = "Axis"
VAL_BINARY = "Binary"
VAL_BLOCK_MATCHING = "BlockMatching"
VAL_BOOLEAN = "Boolean"
VAL_BOTTOM = "Bottom"
VAL_BOX = "Box"
VAL_BUFFER = "Buffer"
VAL_CONSOLE = "Console"
VAL_CONTINUOUS = "Continuous"
VAL_CORRELATION = "Correlation"
VAL_CUBE = "Cube"
VAL_CUBOID = "Cuboid"
VAL_CYLINDER = "Cylinder"
VAL_DHCP = "DHCP"
VAL_DEBUG = "Debug"
VAL_DEBUG_OUT = "DebugOut"
VAL_DELAYED = "Delayed"
VAL_DIAGONAL = "Diagonal"
VAL_DISCARD_NEW = "DiscardNew"
VAL_DISCARD_OLD = "DiscardOld"
VAL_DOWN = "Down"
VAL_ENSENSO = "Ensenso"
VAL_ETHERNET = "Ethernet"
VAL_EUCLIDEAN = "Euclidean"
VAL_FALLING_EDGE = "FallingEdge"
VAL_FILE = "File"
VAL_FIXED = "Fixed"
VAL_FLEX_VIEW2 = "FlexView2"
VAL_FLEXIBLE = "Flexible"
VAL_FLOAT = "Float"
VAL_FORCE = "Force"
VAL_GRID_HEIGHT = "GridHeight"
VAL_GRID_SPACING = "GridSpacing"
VAL_GRID_WIDTH = "GridWidth"
VAL_HALCON = "Halcon"
VAL_HAND = "Hand"
VAL_HIDDEN = "Hidden"
VAL_HIGH = "High"
VAL_HIGH_ACTIVE = "HighActive"
VAL_IMAGE_POSITION = "ImagePosition"
VAL_IMMEDIATE = "Immediate"
VAL_IN_USE = "InUse"
VAL_INDEX = "Index"
VAL_INFO = "Info"
VAL_ITEM = "Item"
VAL_LEFT = "Left"
VAL_LEFT_TO_RIGHT = "LeftToRight"
VAL_LINK = "Link"
VAL_LINK_HIDDEN = "LinkHidden"
VAL_LOCKED = "Locked"
VAL_LOW = "Low"
VAL_LOW_ACTIVE = "LowActive"
VAL_MASK = "Mask"
VAL_MONOCULAR = "Monocular"
VAL_MOVING = "Moving"
VAL_NETWORK_TYPE_A = "NetworkTypeA"
VAL_NETWORK_TYPE_B = "NetworkTypeB"
VAL_NETWORK_TYPE_C = "NetworkTypeC"
VAL_NEW = "New"
VAL_NONE = "None"
VAL_NOT_SPECIFIED = "NotSpecified"
VAL_NULL = "Null"
VAL_NUMBER = "Number"
VAL_OBJECT = "Object"
VAL_OFF = "Off"
VAL_OPEN = "Open"
VAL_ORIGIN = "Origin"
VAL_PWM = "PWM"
VAL_PATTERN = "Pattern"
VAL_PERSISTENT = "Persistent"
VAL_PLANE = "Plane"
VAL_PROJECTOR = "Projector"
VAL_RANDOM = "Random"
VAL_RAW = "Raw"
VAL_RECTIFIED = "Rectified"
VAL_REPROJECTION_ERROR = "ReprojectionError"
VAL_RIGHT = "Right"
VAL_RIGHT_TO_LEFT = "RightToLeft"
VAL_RISING_EDGE = "RisingEdge"
VAL_SERIAL_NUMBER = "SerialNumber"
VAL_SGM_ALIGNED = "SgmAligned"
VAL_SGM_ALIGNED_AND_DIAGONAL = "SgmAlignedAndDiagonal"
VAL_SGM_DIAGONAL = "SgmDiagonal"
VAL_SINGLE = "Single"
VAL_SINGLE_CUSTOM = "SingleCustom"
VAL_SOFTWARE = "Software"
VAL_SPHERE = "Sphere"
VAL_STANDARD = "Standard"
VAL_STATIC = "Static"
VAL_STEREO = "Stereo"
VAL_STRING = "String"
VAL_STRUCTURE_LOCKED = "StructureLocked"
VAL_SUCCESSFUL = "Successful"
VAL_T_PIECE = "TPiece"
VAL_TILT_DIRECTION = "TiltDirection"
VAL_TOP = "Top"
VAL_TRACE = "Trace"
VAL_TRIGGERED = "Triggered"
VAL_TYPE = "Type"
VAL_USB = "USB"
VAL_UNKNOWN = "Unknown"
VAL_UNTRIGGERED = "Untriggered"
VAL_UP = "Up"
VAL_VALIDATE = "Validate"
VAL_VIRTUAL = "Virtual"
VAL_WORKSPACE = "Workspace"
VAL_X = "X"
VAL_XYZ = "XYZ"
VAL_Y = "Y"
VAL_Z = "Z"
VAL_ZYX = "ZYX"

CONSTANTS_PREFIX = {'Commands': 'cmd', 'Errors': 'err', 'Items': 'itm',
                    'Values': 'val', 'ApiErrors': 'NxLib', 'ItemTypes': 'NxLib'}

GET_CONSTANTS_CMD = "GetConstants"


def _update_constants_module():
    try:
        cmd = _NxLibCommand(GET_CONSTANTS_CMD)
        cmd.execute()
        result = cmd.result()

        itm = result.as_json()
        json_object = json.loads(itm)

        for constant_type in json_object:
            if (isinstance(json_object[constant_type], list)):
                prefix = CONSTANTS_PREFIX[constant_type]
                for constant in json_object[constant_type]:
                    variable_name = None
                    value = None
                    if isinstance(constant, dict):
                        variable_name = prefix + constant['Name']
                        value = constant['Value']
                    else:
                        variable_name = prefix + constant
                        value = str(constant)
                    variable_name = _convert_camel_to_upper_snake(variable_name)
                    if variable_name.startswith('NX_LIB'):
                        variable_name = _fix_nxlib_prefix(variable_name)
                    globals()[variable_name] = value
    except:
        raise _NxLibError("Could not load current ensenso_nxlib constants. "
                          "It may be that your ensenso_nxlib version does not support updating.")


try:
    _update_constants_module()
except _NxLibError:
    pass
except:
    pass



# -*- coding: utf-8 -*-

import os
import platform
import re

from .exception import NxLibError


def find_and_return_lib(possible_directories, library_name):
    for possible_directory in possible_directories:
        for file in os.listdir(possible_directory):
            if file == library_name:
                return os.path.join(possible_directory, file)


def _get_os():
    operating_systems = {"win": "Windows",
                         "linux": "Linux",
                         "mac": "Darwin"}

    detected_os = platform.platform(terse=True)
    for possible_os in operating_systems:
        if operating_systems[possible_os] in detected_os:
            return possible_os


def _get_architecture_bits():
    [bits, linkage] = platform.architecture()
    return bits


def get_lib_name(is_remote_lib=False):
    if is_remote_lib:
        default_lib_names = {"win32bit": "NxLibRemote32.dll",
                             "win64bit": "NxLibRemote32.dll",
                             "linux32bit": "libNxLibRemote32.so",
                             "linux64bit": "libNxLibRemote64.so"}
    else:
        default_lib_names = {"win32bit": "NxLib32.dll",
                             "win64bit": "NxLib64.dll",
                             "linux32bit": "libNxLib32.so",
                             "linux64bit": "libNxLib64.so"}
    os = _get_os()
    bits = _get_architecture_bits()

    try:
        return default_lib_names[os + bits]
    except:
        raise NxLibError("Could not determine NxLib library for this "
                         "system os: {} and bits: {}".format(os, bits))


def fix_string_encoding(path):
    try:
        if path is None:
            return b''
        path = path.encode()
    except AttributeError:
        pass
    return path


def convert_camel_to_lower_snake(name):
    s1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()


def convert_camel_to_upper_snake(name):
    s1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s1).upper()


def fix_nxlib_prefix(word):
    return word[:2] + word[3:]



# -*- coding: utf-8 -*-

from .exception import NxLibException
from .item import NxLibItem

__all__ = ["NxLibCommand"]

ITM_EXECUTE = "Execute"
ITM_RESULT = "Result"
ITM_COMMAND = "Command"
ITM_PARAMETERS = "Parameters"
ITM_ERROR_SYMBOL = "ErrorSymbol"

NXLIB_OPERATION_SUCCEEDED = 0
NXLIB_EXECUTION_FAILED = 17
NXLIB_ITEM_TYPE_NULL = 1


class NxLibCommand:
    def __init__(self, command_name, node_name=None):
        self._command_name = command_name
        self._remove_slot_on_destruction = False

        if node_name is None:
            self.create_temporary_slot()
        else:
            self._command_item = NxLibItem()[ITM_EXECUTE][node_name]

    def __del__(self):
        try:
            if self._remove_slot_on_destruction:
                self._command_item.erase()
        except NxLibException:
            pass

    def create_temporary_slot(self, base_name=None):
        self._command_item = NxLibItem()[ITM_EXECUTE].make_unique_item(base_name)
        self._remove_slot_on_destruction = True

    def slot(self):
        return self._command_item

    def parameters(self):
        return self.slot()[ITM_PARAMETERS]

    def result(self):
        return self.slot()[ITM_RESULT]

    def successful(self):
        return not self.result()[ITM_ERROR_SYMBOL].exists()

    def execute(self, command_name=None, wait=True):
        if not command_name:
            command_name = self._command_name

        function_item = self.slot()[ITM_COMMAND]
        function_item.set_t(command_name)

        if wait:
            function_item.wait_for_type(NXLIB_ITEM_TYPE_NULL, True)
            self.assert_successful()

    def finished(self):
        return not self.slot()[ITM_COMMAND].exists() or self.slot()[ITM_COMMAND].is_null()

    def assert_successful(self):
        if not self.finished() or not self.successful():
            raise NxLibException(self.slot().path, NXLIB_EXECUTION_FAILED, self)



from setuptools import setup
from os import path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(name='ensenso_nxlib',
      packages=['ensenso_nxlib'],
      python_requires='>3.5.0',
      version='0.1.post1',
      description='Python interface to interact with the Ensenso NxLib',
      long_description=long_description,
      long_description_content_type='text/markdown',
      classifiers=[
          'Development Status :: 4 - Beta',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python :: 3'
      ],
      url='https://github.com/ensenso/nxlib-python-interface.git',
      author='Yasin Guenduez, Paul Rogister',
      author_email='yasin.guenduez@ensenso.com, paul.rogister@isys-vision.de',
      license='MIT',
      install_requires=[
          'numpy'
      ],
      setup_requires=['pytest-runner', 'flake8', 'pep8-naming'],
      tests_require=['pytest'],
      zip_safe=False)



import ensenso_nxlib

__all__ = [
    "NxLibError",
    "NxLibException"
]


class NxLibError(Exception):
    def __init__(self, message=None):
        self._message = message

    def __str__(self):
        return self._message


class NxLibException(NxLibError):
    def __init__(self, path, error_code, command=None):
        self._path = path
        self._error_code = error_code

        # Save command object in the exception to keep temporary slots alive while an exception exists.
        self._command = command

    def get_error_code(self):
        return self._error_code

    def get_error_text(self):
        return ensenso_nxlib.api.translate_error_code(self._error_code)

    def get_item_path(self):
        return self._path

    def __str__(self):
        message = "NxLib error {} ({}) for item {}".format(self._error_code, self.get_error_text(), self._path)

        try:
            if self._path:
                message += "\nCurrent item value: {}".format(ensenso_nxlib.NxLibItem(self._path).as_json(True))
        except NxLibError:
            pass

        return message
